#!/bin/bash 

AudioBitrate=384
OverheadBitrate=200
DVDSize=4400000000

ReadConfig ()

{

 echo "Encoder configuration:"  >> DebugOutput.txt
 echo ""  >> DebugOutput.txt
 cat "$1" >> DebugOutput.txt
 echo ""  >> DebugOutput.txt

 MPEGOptimize=$(cat "$1" | grep -E ^MPEGOptimize= | cut -d \" -f 2)
 EncodingMethod=$(cat "$1" | grep -E ^EncodingMethod= | cut -d \" -f 2)
 MPEGBitrate=$(cat "$1" | grep -E ^Bitrate= | cut -d \" -f 2)

 MPEGVideoBitrate=$[ $MPEGBitrate - $AudioBitrate - $OverheadBitrate ]

 Quantiser=$(cat "$1" | grep -E ^Quantizer= | cut -d \" -f 2)
 NoHighFrequencies=$(cat "$1" | grep -E ^NoHighFrequencies= | cut -d \" -f 2)
 FinalUnsharp=$(cat "$1" | grep -E ^FinalUnsharp= | cut -d \" -f 2)
 MildDenoise=$(cat "$1" | grep -E ^MildDenoise= | cut -d \" -f 2)
 HalfRes=$(cat "$1" | grep -E ^HalfRes= | cut -d \" -f 2)
 QuantMatrix=$(cat "$1" | grep -E ^QuantMatrix= | cut -d \" -f 2)
 CustomCommandlineSwitches="$(cat "$1" | grep -E ^CustomCommandlineSwitches= | cut -d \" -f 2)"
 StoragePath="$(cat "$1" | grep -E ^StoragePath= | cut -d \" -f 2)"
 OutputFileName="$(cat "$1" | grep -E ^OutputFileName= | cut -d \" -f 2)"
 ColorCorrection=$(cat "$1" | grep -E ^ColorCorrection= | cut -d \" -f 2)
 Monochrome=$(cat "$1" | grep -E ^Monochrome= | cut -d \" -f 2)
 ProgressMonitor=$(cat "$1" | grep -E ^ProgressMonitor= | cut -d \" -f 2)
 ContrastValue=$(cat "$1" | grep -E ^ContrastValue= | cut -d = -f 2)
 BrightnessValue=$(cat "$1" | grep -E ^BrightnessValue= | cut -d = -f 2)
 SaturationValue=$(cat "$1" | grep -E ^SaturationValue= | cut -d = -f 2)
} 

# Report error and abort script

PrintError ()
{
 X=$(type -p kdialog)

 if test $X && test -x $X && [ -n "$DISPLAY" ]; then

  kdialog --error "$1" 

 else

  echo >> DebugOutput.txt
  echo "---- RECEIVED ERROR MESSAGE ----" >> DebugOutput.txt
  echo >> DebugOutput.txt
  echo -e "$1" >> DebugOutput.txt
  echo >> DebugOutput.txt
  echo "--------------------------------" >> DebugOutput.txt

 fi

 exit 1
}

# Present a warning message to the user
# Argument $1 is warning message.

PrintWarning ()
{
 X=$(type -p kdialog)

 if test $X && test -x $X && [ -n "$DISPLAY" ]; then
  kdialog --warningcontinuecancel "$1" >> DebugOutput.txt
  if [ ! "$?" -eq 0 ]; then
  echo >> DebugOutput.txt
  echo "USER ABORTED PROGRAM" >> DebugOutput.txt
  echo >> DebugOutput.txt
  exit 1
  fi
  
 else
  echo >> DebugOutput.txt
  echo "---- WARNING ----" >> DebugOutput.txt
  echo >> DebugOutput.txt
  echo -e "$1" >> DebugOutput.txt
  echo >> DebugOutput.txt
  echo "-----------------" >> DebugOutput.txt
 fi
}


# ----------------------------------------------------------
# DISK SPACE CHECKING ROUTINES
# ----------------------------------------------------------

# Warn if disk space in directory $1 lower than $2 MB

DiskSpaceWarning ()
{
  FreeSpace=$(df  -m "$1" -P | awk '{print $4}' | tail -n 1)

  if [ $FreeSpace -lt $2 ]; then

    PrintWarning "You are low on disk space in this folder:\n\n$1\n\nYou are strongly encouraged to free some disk space before continuing."

    if [ "$?" -eq 1 ]; then
      exit 1
    fi

  fi
}

# Abort if disk space ran out in directory $1

DiskSpaceError ()
{
  FreeSpace=$(df  -m "$1" -P | awk '{print $4}' | tail -n 1)

  if [ $FreeSpace -le 1 ]; then
    PrintError "You ran out of disk space in this folder:\n\n$1\n\nPlease free more disk space and try again."
    exit 1
  fi
}


CompileCLIOptions ()

{
 # Adjust color ranges to Rec.601 specification
 if [ $Monochrome == "true" ]; then

   StandardColorCorrection="| yuvcorrect -Y LUMINANCE_1.0_16_235_16_235 -Y CHROMINANCE_0.0_1.0_128_1.0_128_128_128"

 else

   StandardColorCorrection="| yuvcorrect -Y LUMINANCE_1.0_16_235_16_235 -Y CHROMINANCE_0.0_1.0_128_1.0_128_16_240"

 fi

 # Brightness/Contrast/Saturation Correction using yuvcorrect

 if [ $ColorCorrection == "true" ]; then

   # Do full gamma, contrast and saturation correction
  
   if [ $ContrastValue -ge 0 ]; then
  
     # Increase contrast
  
     YUVCorrectCommand="| yuvcorrect -Y LUMINANCE_`echo "scale=2; ($BrightnessValue * 2 ) / 100" | bc`_${ContrastValue}_`echo $[ 255 - $ContrastValue ]`_0_255 -Y CHROMINANCE_0.0_`echo "scale=2; ($SaturationValue * 2 ) / 100" | bc`_128_`echo "scale=2; ($SaturationValue * 2 ) / 100" | bc`_128_0_255"
    
   else
  
     # Decrease contrast
  
     YUVCorrectCommand="| yuvcorrect -Y LUMINANCE_`echo "scale=2; ($BrightnessValue * 2 ) / 100" | bc`_0_255_$(( 0 - $ContrastValue))_$(( 255 + $ContrastValue )) -Y CHROMINANCE_0.0_`echo "scale=2; ($SaturationValue * 2 ) / 100" | bc`_128_`echo "scale=2; ($SaturationValue * 2 ) / 100" | bc`_128_0_255"
  
   fi

 else

   YUVCorrectCommand=""

 fi

 if [ "$MPEGOptimize" -ge 1 ]; then 

   MPEGOptimizeOptions="-r 32"

   if [ "$MPEGOptimize" -ge 2 ]; then

     MPEGOptimizeOptions="-r 32 -4 2 -2 1"

   fi

 fi  

 # Suppress high frequencies to reduce mild noise

 if [ $NoHighFrequencies == "true" ]; then
   NoHighFrequenciesOption="-N 1.0"
 fi

 # A mild median filter just before entering the 
 # encoder is a good idea, most of the time.
 # This filter can save up to 10% bitrate.

 if [ $MildDenoise == "true" ]; then
  DenoiseCommand="| yuvmedianfilter -f -R 1 -r 1 -w 20"
 fi

 if [ $HalfRes == "true" ]; then

   #HalfResCommand="| yuvscaler -O SIZE_352x576"
    HalfResCommand="| y4mscaler -O size=352x576"

 fi

 # Use correct aspect ratio for MPEG2 encoding

 if [ $TargetAspect == "wide" ] || [ $TargetAspect == "letterboxed" ]; then
  MPEGAspectOption="-a 3"
 else
  MPEGAspectOption="-a 2"
 fi
} 
 
Encode ()
{

 if [ $EncodingMethod == "VBR" ]; then

   QuantiserOption="-q $Quantiser"

 else

   CBROption="--cbr"

 fi

 # Change framerate when needed

  # Determine what output framerate we need

  if [ $TargetNorm == "PAL" ]; then

    TargetFrameRate="25:1"

  else 

    if [ "$InputFrameRate" == "30000:1001" ] || [ "$InputFrameRate" == "24000:1001" ]; then

      TargetFrameRate=$InputFrameRate

    else

      TargetFrameRate="30000:1001"

    fi

  fi

 if [ ! "$InputFrameRate" == "$TargetFrameRate" ]; then

  echo "Framerate will be converted from ${InputFrameRate} to ${TargetFrameRate}." >> DebugOutput.txt

  ChangeFPSCommand="| yuvfps -n -r $TargetFrameRate"

 fi

 if [ "$FinalUnsharp" == "true" ]; then

   echo "We will do a final unsharp masking pass." >> DebugOutput.txt
   FinalUnsharpCommand="| y4munsharp -L .1,.96,0"

 fi

 echo ".." >> DebugOutput.txt
 echo "Encoding method is $EncodingMethod" >> DebugOutput.txt
 echo ".." >> DebugOutput.txt

 if [ "$ProgressMonitor" == "true" ]; then

   fifofile="EncoderPipe_$RANDOM"

   mkfifo "$fifofile"

   EncoderCommand="cat /dev/stdin $YUVCorrectCommand $StandardColorCorrection $DenoiseCommand $ClipCommand $FinalUnsharpCommand $HalfResCommand $ChangeFPSCommand | tee \"$fifofile\" | mpeg2enc -M 0 -f 8 -b $MPEGVideoBitrate -B $[$AudioBitrate + $OverheadBitrate] -D 10 -K $QuantMatrix $QuantiserOption $CBROption $MPEGAspectOption $MPEGOptimizeOptions $NoHighFrequenciesOption $CustomCommandlineSwitches -o \"$StoragePath/${OutputFileName}.m2v\" > \"mpeg2enc-output.txt\" 2>&1"

   cat "$fifofile" | yuvplay &

   echo "YUVEncoder COMMAND: $EncoderCommand" >> DebugOutput.txt
   echo "Begin Encoding..." >> DebugOutput.txt

   eval $EncoderCommand

   echo "Encoding process ended." >> DebugOutput.txt

 else

   EncoderCommand="cat /dev/stdin $YUVCorrectCommand $StandardColorCorrection $DenoiseCommand $ClipCommand $FinalUnsharpCommand $HalfResCommand $ChangeFPSCommand | mpeg2enc -M 0  -f 8 -b $MPEGVideoBitrate -B $AudioBitrate -D 10 -K $QuantMatrix $QuantiserOption $CBROption $MPEGAspectOption $MPEGOptimizeOptions $NoHighFrequenciesOption $CustomCommandlineSwitches -o \"$StoragePath/${OutputFileName}.m2v\" > \"mpeg2enc-output.txt\" 2>&1"

   echo "YUVEncoder COMMAND: $EncoderCommand" >> DebugOutput.txt
   echo "Begin Encoding..." >> DebugOutput.txt

   eval $EncoderCommand

   echo "Encoding process ended." >> DebugOutput.txt

 fi

 DiskSpaceError "$StoragePath"

 Quantizers="$(cat "mpeg2enc-output.txt" | grep mpeg2enc | grep -E -o q=[0-9]{1,\} | cut -d = -f 2)"

 if [ -z "$Quantizers" ]; then

   echo "PROBLEM: mpeg2enc output shows no encoded frames. Something went wrong." >> DebugOutput.txt
   echo "" >> DebugOutput.txt
   echo "Head of mpeg2enc output:" >> DebugOutput.txt
   echo "" >> DebugOutput.txt
   head -n 100 "mpeg2enc-output.txt" >> DebugOutput.txt
   echo "Tail of mpeg2enc output:" >> DebugOutput.txt
   echo "" >> DebugOutput.txt
   tail -n 100 "mpeg2enc-output.txt" >> DebugOutput.txt

   PrintError "No MPEG file was generated!\n\nPlease check that your 'ffmpeg' and your 'mjpegtools' packages are correctly installed and working. Otherwise, please contact the author of KmPg2."

 fi

 Counter=0
 TotalQ=0
 LowestQ=31
 HighestQ=0

 for i in $Quantizers ; do

 TotalQ=$[$TotalQ + $i]
 Counter=$[$Counter + 1]

 if [ $i -lt $LowestQ  ]; then

   LowestQ=$i

 fi

 if [ $i -gt $HighestQ  ]; then

   HighestQ=$i

 fi

 done

 MeanQ="$(echo "scale=2; $TotalQ / $Counter" | bc)"
 
 echo "Encoding quality ranged from $(echo "scale=1; 100 - ( ($HighestQ * 100) / 31 )" | bc)% upto `echo "scale=1; 100 - ( ($LowestQ * 100) / 31 )" | bc`%. Average quality: $(echo "scale=1; 100 - ( ($MeanQ * 100) / 31 )" | bc)%." > "KmPg2-QualityReport.txt"

 echo "Head of mpeg2enc output:" >> DebugOutput.txt
 echo "" >> DebugOutput.txt
 head -n 100 "mpeg2enc-output.txt" >> DebugOutput.txt
 echo "Tail of mpeg2enc output:" >> DebugOutput.txt
 echo "" >> DebugOutput.txt
 tail -n 100 "mpeg2enc-output.txt" >> DebugOutput.txt
 echo "Quality Report so far:" >> DebugOutput.txt
 echo "" >> DebugOutput.txt
 cat "KmPg2-QualityReport.txt" >> DebugOutput.txt

 rm -f "$fifofile"
 rm -f "mpeg2enc-output.txt"
}


# Program Starts Here

echo "YUVEncoder is starting:" >> DebugOutput.txt

read -u0 TargetAspect
read -u0 TargetNorm
read -u0 InputFrameRate
read -u0 ClipCommand
read -u0 VideoLength

echo "Received info on stdin:" >> DebugOutput.txt
echo "" >> DebugOutput.txt
echo "ClipCommand:     $ClipCommand" >> DebugOutput.txt
echo "Target Aspect:   $TargetAspect" >> DebugOutput.txt
echo "Target Norm:     $TargetNorm" >> DebugOutput.txt
echo "Input Framerate: $InputFrameRate" >> DebugOutput.txt
echo "Video Length: $VideoLength ms" >> DebugOutput.txt
echo "" >> DebugOutput.txt

ReadConfig "$1"
CompileCLIOptions

# Estimate MPEG filesize in bytes

EstimatedFileSize=$[ ( $VideoLength * $MPEGBitrate ) / 8 ]

echo "Estimated final filesize: $[ $EstimatedFileSize / (1024 * 1024) ] MB " >> DebugOutput.txt
echo "Checking if $EstimatedFileSize >=  $DVDSize" >> DebugOutput.txt

if [ $EstimatedFileSize -ge $DVDSize ]; then

  RecommendedBitrate1=$[ ( $DVDSize * 8 ) / $VideoLength ]
  RecommendedBitrate2=$[ ( ( $DVDSize - (100 * 1024 * 1024) ) * 8 ) / $VideoLength ]

  PrintWarning "The new MPEG2 file will probably NOT fit on a DVD disk.\n\nIf you plan to burn the video on a DVD disk, you should lower the bitrate to $RecommendedBitrate1 kbit/s. If you also want to fit a menu on the DVD, choose a bitrate of $RecommendedBitrate2."

fi

Encode

echo "Done. Have a nice day!" >> DebugOutput.txt
