#!/bin/bash

ReadProcessingProfile ()

{
  if [ ! -e "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" ]; then

    echo "Processing Preset $ProcessingProfile not found. Aborting." >> DebugOutput.txt
    exit -1

  fi

  echo "YUVFilter Processing Profile ($ProcessingProfile):"  >> DebugOutput.txt
  echo ""  >> DebugOutput.txt
  cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" >> DebugOutput.txt
  echo ""  >> DebugOutput.txt


  CutOverscanCommand="$( cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" | grep -E ^CutOverscanCommand= | cut -d \" -f 2)" 
  LowPassCommand="$( cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" | grep -E ^LowPassCommand= | cut -d \" -f 2)" 
  DenoiseCommand="$( cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" | grep -E ^DenoiseCommand= | cut -d \" -f 2)" 
  SharpenCommand="$( cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" | grep -E ^SharpenCommand= | cut -d \" -f 2)" 
  ColorCorrection="$( cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" | grep -E ^ColorCorrection= | cut -d \" -f 2)" 
  MonochromeCommand="$( cat "$HOME/.kde/share/apps/KmPg2/$ProcessingProfile" | grep -E ^MonochromeCommand= | cut -d \" -f 2)" 

}


# APPLICATION STARTS HERE

echo "YUVFilter is starting:" >> DebugOutput.txt

ProcessingProfile="$1"

ReadProcessingProfile

# Read info from stdout of decoder

read -u0 InputFrameRate
read -u0 InputSampleAspect
read -u0 InputXRes
read -u0 InputYRes
read -u0 InputSampleAspectX
read -u0 InputSampleAspectY
read -u0 VideoLength

echo "Received info on stdin:" >> DebugOutput.txt
echo "" >> DebugOutput.txt
echo "Input Framerate: $InputFrameRate" >> DebugOutput.txt
echo "Input Sample Aspect: $InputSampleAspect" >> DebugOutput.txt
echo "Input XRes: $InputXRes" >> DebugOutput.txt
echo "Input YRes: $InputYRes" >> DebugOutput.txt
echo "Input SampleAspectX: $InputSampleAspectX" >> DebugOutput.txt
echo "Input SampleAspectY: $InputSampleAspectY" >> DebugOutput.txt
echo "Video Length: $VideoLength ms" >> DebugOutput.txt
echo "" >> DebugOutput.txt

# Add filtering tools to pipeline

FilterCommand="cat /dev/stdin $LowPassCommand $DenoiseCommand $SharpenCommand $ColorCorrection $MonochromeCommand"

# Pass on info to Prescaler

echo "$InputFrameRate"
echo "$InputSampleAspect"
echo "$InputXRes"
echo "$InputYRes"
echo "$InputSampleAspectX"
echo "$InputSampleAspectY"
echo "$VideoLength"

# Generate scaled YUV stream on stdout

echo "YUVFilter COMMAND: $FilterCommand" >> DebugOutput.txt

eval "$FilterCommand"