#!/bin/bash 


#TODO:

# Als je streamcopy doet van AC3 streams, detecteer dan de bitrate van deze stream.
# dit is nodig voor correcte berekening van peak bitrate. Doe dit alleen wanneer er
# GEEN preview gemaakt wordt!
#
# In geval van audio-loze output bitrate nul maken, voor correcte peak bitrate berekening.

AudioBitrate=384
OverheadBitrate=200
DVDSize=4400000000

SourceVideo="$1"
MPEG2File="$2"
TargetBitrate=$3
EncodingMethod="$4"
SubtitleFile="$5"
SubtitleFont="$6"

# ----------------------------------------------------------
# ROUTINES TO COMMUNICATE ERRORS/WARNINGS TO THE USER
# ----------------------------------------------------------

# Present an error message to the user and abort.
# Argument $1 is error message.

PrintError ()
{
 X=`type -p kdialog`

 if test $X && test -x $X && [ -n "$DISPLAY" ]; then

  kdialog --error "$1"

  UpdateTask "Aborted."
  UpdateProgress 0

 else

  echo
  echo "---- RECEIVED ERROR MESSAGE ----"
  echo
  echo -e "$1"
  echo
  echo "--------------------------------"

 fi

 exit 1
}

# Present a warning message to the user
# Argument $1 is warning message.

PrintWarning ()
{
 X=`type -p kdialog`

 if test $X && test -x $X && [ -n "$DISPLAY" ]; then
  kdialog --warningcontinuecancel "$1"
  if [ ! "$?" -eq 0 ]; then
  echo
  echo "USER ABORTED PROGRAM"
  echo
  exit 1
  fi
  
 else
  echo
  echo "---- WARNING ----"
  echo
  echo -e "$1"
  echo
  echo "-----------------"
 fi
}

# ----------------------------------------------------------
# DISK SPACE CHECKING ROUTINES
# ----------------------------------------------------------

# Warn if disk space in directory $1 lower than $2 MB

DiskSpaceWarning ()
{
  FreeSpace=`df  -m "$1" -P | awk '{print $4}' | tail -n 1`

  if [ $FreeSpace -lt $2 ]; then

    PrintWarning "You are low on disk space in this folder:\n\n$1\n\nYou are strongly encouraged to free some disk space before continuing."

    if [ "$?" -eq 1 ]; then
      exit 1
    fi

  fi
}

# Abort if disk space ran out in directory $1

DiskSpaceError ()
{
  FreeSpace=`df  -m "$1" -P | awk '{print $4}' | tail -n 1`

  if [ $FreeSpace -le 1 ]; then
    PrintError "You ran out of disk space in this folder:\n\n$1\n\nPlease free more disk space and try again."
    exit 1
  fi
}


# Probe audio sampling frequency of file $1 track $2
# $1 = audio input
# $2 = audio track

DetectAudioFreq ()
{
    echo "`ffmpeg -i "$1" 2>&1 | \
    awk '/Audio:/ {print $5}' | sed s/,// | head -n ${2} | tail -n 1`"
}

# Probe audio format of file $1 track $2
# $1 = audio input
# $2 = audio track

DetectAudioFormat ()
{
    echo "`ffmpeg -i "$1" 2>&1 | \
    awk '/Audio:/ {print $4}' | sed s/,// | head -n ${2} | tail -n 1`"
}

# Detects length of file $1
# Outputs number of milliseconds

DetectVideoLength ()
{
  Hours=$(ffmpeg -i "$1" 2>&1 | grep Duration | awk '{print $2}' | cut -d : -f 1)
  Minutes=$(ffmpeg -i "$1" 2>&1 | grep Duration | awk '{print $2}' | cut -d : -f 2)
  Seconds=$(ffmpeg -i "$1" 2>&1 | grep Duration | awk '{print $2}' | cut -d : -f 3 | cut -d , -f 1)

  echo "$Hours*3600000 + $Minutes*60000 + $Seconds*1000" | bc | cut -d . -f 1 | cut -d , -f 1
}

# Probe resolution of file $1.
# Output is of the form "640x480"

DetectVideoResolution ()
{
    ffmpeg -i "$1" -f yuv4mpegpipe -pix_fmt yuv420p -t 1 -y /tmp/tmp.y4m >/dev/null 2>&1
    echo "`head -n 1 /tmp/tmp.y4m | awk '{sub("W","",$2); sub("H","",$3); print $2"x"$3}'`"
    rm -f /tmp/tmp.y4m
}

OutputFileName="$(echo "$MPEG2File" | cut -d . -f 1).mpg"
QualityReportFile="$(pwd)/KmPg2-QualityReport.txt"


# Multiplex audio

echo "Multiplexing..."

echo "SourceVideo is $SourceVideo"
echo "MPEG2File is $MPEG2File"

DirName="$(dirname "$MPEG2File")"
FreeSpace=$(df -m "$DirName" -P | awk '{print $4}' | tail -n 1)
RequiredSpace=$[ ( $(du -m "$MPEG2File" | awk '{print $1}') * 120 ) /100 ]

echo "Free disk space is $FreeSpace MB, Required space is $RequiredSpace MB."

InputAudioFormat=$(DetectAudioFormat "$SourceVideo" 1)

echo "Detected audio format: $InputAudioFormat"

InputVideoLength=$(DetectVideoLength "$SourceVideo")

if [ -z "$InputVideoLength" ]; then

  InputVideoLength=1
  echo "WARNING: Video length could not be detected."

else

  echo "Detected video length is $InputVideoLength ms."

fi

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#
# Check of ffmpeg audiotrack kan decoderen!!
# Zo niet: Multiplex stilte!
#
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

if [ "$SourceVideo" == "NONE" ]; then

  mplex -f 8 -V "$MPEG2File" -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

else

  if [ -z "$InputAudioFormat" ]; then

    echo "WARNING: Input video has no audio track. Muxing without audio." >> DebugOutput.txt

    # Generate 48000 samples per second
    #NumAudioFrames=$[ $InputVideoLength * 48 ]

    #dd if=/dev/zero bs=2 count=$NumAudioFrames | sox -r 48000 -t .sw - -t .wav -w /dev/stdout | ffmpeg -i /dev/stdin -acodec copy -y -f ac3 /dev/stdout | mplex -f 8 -V "$MPEG2File" /dev/stdin -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    mplex -f 8 -V "$MPEG2File" -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    echo "During Sound-less multiplex attempt, MPlex Said (tail):"

    cat "MPlexOutput.txt" >> DebugOutput.txt

  elif [ -n "`echo "$InputAudioFormat" | grep 0x`" ]; then

    # Input video has unknown audio stream. Multiplex without audio.

    mplex -f 8 -V "$MPEG2File" -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    echo "During Sound-less multiplex attempt, MPlex Said (tail):"

    cat "MPlexOutput.txt" | tail -n 100

  elif [ "$InputAudioFormat" == "ac3" ]; then

    # Input video has AC3 audio stream. We can simply copy it to the output.

    ffmpeg -i "$SourceVideo" -acodec copy -y -f ac3 /dev/stdout | mplex -f 8 -V "$MPEG2File" /dev/stdin -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    echo "During AC3 Stream-Copy Multiplexing, MPlex Said (tail):"

    cat "MPlexOutput.txt" | tail -n 100

  else

    InputAudioFreq=`DetectAudioFreq "$SourceVideo" 1`

    echo "Detected audio sampling frequency of $InputAudioFreq"

    if [ ! $InputAudioFreq == 48000 ]; then

      echo "Using Sox to convert to 48 KHz." >> DebugOutput.txt

      ffmpeg -i "$SourceVideo" -v 0 -f au -y /dev/stdout | sox -t au /dev/stdin -r 48000 -t au /dev/stdout polyphase | ffmpeg -f au -i /dev/stdin -ab $AudioBitrate -ar 48000 -ac 2 -y -f ac3 /dev/stdout | mplex -f 8 -V "$MPEG2File" /dev/stdin -o "$OutputFileName" > "MPlexOutput.txt" 2>&1 

    else

      echo "No samplerate conversion needed." >> DebugOutput.txt

      ffmpeg -i "$SourceVideo" -ab $AudioBitrate -ar 48000 -ac 2 -y -f ac3 /dev/stdout | mplex -f 8 -V "$MPEG2File" /dev/stdin -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    fi

  fi

fi


echo "Tail of MPlex Output:" 

cat "MPlexOutput.txt" | tail -n 100

if [ ! -z "`cat "MPlexOutput.txt" | grep ERROR`" ]; then

 PrintError "Multiplexing failed. MPlex final words were:\n\n`cat "MPlexOutput.txt" | tail -n 10`"

fi


if [ -n "`cat "MPlexOutput.txt" | grep 'Unsupported codec'`" ]; then

  # The audio codec is not supported

  if [ "$InputAudioFormat" == "0x2000" ]; then

    # Input audio is AC3, but FFMPEG version has no support for it.
    # Simply pipe the input stream straight to mplex.

    ffmpeg -i "$SourceVideo" -acodec copy -y -f ac3 /dev/stdout | mplex -f 8 -V "$MPEG2File" /dev/stdin -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    echo "During Stream-Copy attempt, MPlex Said (tail):" >> "DebugOutput.txt"

    cat "MPlexOutput.txt" | tail -n 100

  else

    PrintWarning "The audio format of your video is not supported by your FFMPEG version. This means that it cannot be converted to a format that is DVD compatible.\n\nThe soundtrack of the generated MPEG2 file will be silent."

    # Generate 48000 samples per second
    #NumAudioFrames=$[ $InputVideoLength * 48 ]

    #dd if=/dev/zero bs=2 count=$NumAudioFrames | sox -r 48000 -t .sw - -t .wav -w /dev/stdout | ffmpeg -i /dev/stdin -acodec copy -y -f ac3 /dev/stdout | mplex -f 8 -V "$MPEG2File" /dev/stdin -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    mplex -f 8 -V "$MPEG2File" -o "$OutputFileName" > "MPlexOutput.txt" 2>&1

    echo "During Sound-less multiplex attempt, MPlex Said (tail):"

    cat "MPlexOutput.txt"

  fi

fi

# Check if we ran out of disk space

DiskSpaceError "`dirname "$MPEG2File"`"

# Multiplexing is finished now, remove m2v file.

rm -f "$MPEG2File"

# Check if we need to multiplex subtitles as well

if [ -n "$SubtitleFile" ] && [ -e "$SubtitleFile" ] && [ -e "$SubtitleFont" ]; then

  SPUMuxFile="`dirname "$MPEG2File"`/SPUMux.xml"
  TempFontFileName="KmPg2_$RANDOM"

  mkdir "$HOME/.spumux"
  cp "$SubtitleFont" "$HOME/.spumux/$TempFontFileName"


  VideoResolution=$(DetectVideoResolution "$OutputFileName")

  VideoWidth=$(echo "$VideoResolution" | cut -d x -f 1)
  VideoHeight=$(echo "$VideoResolution" | cut -d x -f 2)

  echo "In preparation of SPUMux, detected video resolution of $VideoResolution."

  echo "<subpictures>" > "$SPUMuxFile" 
  echo "  <stream>" >> "$SPUMuxFile" 
  echo "    <textsub filename=\"$SubtitleFile\" characterset=\"ISO8859-1\"" >> "$SPUMuxFile"  
  echo "      fontsize=\"24.0\" font=\"$TempFontFileName\" horizontal-alignment=\"center\"" >> "$SPUMuxFile"  
  echo "      vertical-alignment=\"bottom\" left-margin=\"60\" right-margin=\"60\"" >> "$SPUMuxFile"  
  echo "      top-margin=\"20\" bottom-margin=\"30\"" >> "$SPUMuxFile"
  echo "      movie-width=\"$VideoWidth\" movie-height=\"$[$VideoHeight - 4]\"" >> "$SPUMuxFile"
  echo "      />" >> "$SPUMuxFile" 
  echo "  </stream>" >> "$SPUMuxFile" 
  echo "</subpictures>" >> "$SPUMuxFile" 

  echo "Multiplexing subtitles using this SPUMux XML file:"
  cat "$SPUMuxFile"

  PrevDir="$(pwd)"  

  pushd "`dirname "$MPEG2File"`"
  spumux SPUMux.xml < "$OutputFileName" > "$OutputFileName.spumux"

  SPUMuxErrorCode=$?

  DiskSpaceError "`dirname "$MPEG2File"`"

  rm -f "$HOME/.spumux/$TempFontFileName" "SPUMux.xml"

  if [ $SPUMuxErrorCode -eq 0 ]; then

    mv "$OutputFileName.spumux" "$OutputFileName"

  else

    rm -f "$OutputFileName.spumux"
    PrintWarning "Adding subtitles failed. The final video will not contain any subtitles."

  fi

  popd


fi

AverageBitrate=`cat "MPlexOutput.txt" | grep  "Average bit-rate" | grep -E -o [0-9]{1,\}\ {1,\}bits/sec | awk '{print $1}'`
PeakBitrate=`cat "MPlexOutput.txt" | grep  "Peak bit-rate" | grep -E -o [0-9]{1,\}\ {1,\}bits/sec | awk '{print $1}'`

# Compute total bitrate statistics of stream, video + audio + MUX overhead

AverageBitrate=$[ $AverageBitrate + ($AudioBitrate * 1000) + ($OverheadBitrate * 1000) ]
PeakBitrate=$[ $PeakBitrate + ($AudioBitrate * 1000) + ($OverheadBitrate * 1000) ]

if [ ! -e "$OutputFileName" ]; then
  PrintError "Multiplexing failed.\n\n"
fi

FileSize=`du -b "$OutputFileName" | awk '{print $1}'`

echo -e "\nAverage Bitrate: $(($AverageBitrate / 1000)) KBit/s\nPeak Bitrate: $(($PeakBitrate / 1000)) KBit/s" >> "$QualityReportFile"

if [ ! "$SourceVideo" == "NONE" ]; then

  echo -e "\n\nThe size of the generated MPEG2 file is $[$FileSize / (1024*1024)] MB, which equals to `echo "scale=1; ($FileSize * 100) / $DVDSize" | bc`% of the available space on a DVD disk." >> "$QualityReportFile"

fi

if [ $(($PeakBitrate / 1000)) -ge 8400 ]; then

  if [ "$SourceVideo" == "NONE" ]; then
    echo "" >> "$QualityReportFile"
    echo "" >> "$QualityReportFile"
    echo -e "WARNING: The peak bitrate of the encoded part of the video is $(( ($PeakBitrate / 1000) + $AudioBitrate )) KBit/s. Some DVD players cannot handle bitrates above 8400 KBit/s. Playback may stutter, or even stop.\n\nUse a better processing profile, lower quality settings, a lower bitrate, or try Constant Bitrate encoding." >> "$QualityReportFile"
  else
    echo "" >> "$QualityReportFile"
    echo "" >> "$QualityReportFile"
    echo -e "WARNING: The peak bitrate of the encoded part of the video is $(( ($PeakBitrate / 1000) + $AudioBitrate )) KBit/s. Some DVD players cannot handle bitrates above 8400 KBit/s. Playback may stutter, or even stop.\n\nUse a better processing profile, lower quality settings, a lower bitrate, or try Constant Bitrate encoding." >> "$QualityReportFile"
  fi

fi

if [ $(( ( $PeakBitrate * 100 ) / ( $TargetBitrate * 1000) )) -lt 80 ]; then

  if [ "$SourceVideo" == "NONE" ]; then

    echo "" >> "$QualityReportFile"
    echo "" >> "$QualityReportFile"
    echo -e "HINT: The peak bitrate of the encoded part of the video ($(($PeakBitrate / 1000))) is lower than the bitrate you requested ($TargetBitrate).\n\nYou may want to try using higher quality settings, for example by using the 'Hi-Res' quantizer matrix." >> "$QualityReportFile"

  else

    echo "" >> "$QualityReportFile"
    echo "" >> "$QualityReportFile"
    echo -e "HINT: The peak bitrate ($(($PeakBitrate / 1000))) was lower than the bitrate you requested ($TargetBitrate).\n\nYou may want to try using higher quality settings, for example by using the 'Hi-Res' quantizer matrix." >> "$QualityReportFile"

  fi

fi

echo "$(( ($PeakBitrate * 100 ) / $AverageBitrate )) lower than 110 and $(( ( $PeakBitrate * 100 ) / ($TargetBitrate * 1000) )) greater than 80"

if [ $(( ($PeakBitrate * 100 ) / $AverageBitrate )) -lt 110 ] && [ $(( ( $AverageBitrate * 100 ) / ($TargetBitrate * 1000) )) -ge 80 ] && [ $(( ( $PeakBitrate * 100 ) / ( $TargetBitrate * 1000))) -ge 80 ] && [ "$EncodingMethod" == "VBR" ]; then

  if [ "$SourceVideo" == "NONE" ]; then
    echo "" >> "$QualityReportFile"
    echo "" >> "$QualityReportFile"
    echo -e "WARNING: The average bitrate ($(($AverageBitrate / 1000))) of this part of the video is very close to the peak bitrate ($(($PeakBitrate / 1000))). Fast movements may cause inferior image quality. \n\nOther parts of this video that contain more fast motion than this preview does will almost certainly look bad.\n\nTry using lower quality settings, or use a better processing profile. You may also want to try using constant bitrate encoding." >> "$QualityReportFile"
  else
    echo "" >> "$QualityReportFile"
    echo "" >> "$QualityReportFile"
    echo -e "WARNING: The average bitrate ($(($AverageBitrate / 1000))) was very close to the peak bitrate ($(($PeakBitrate / 1000))). Fast movements may have caused inferior image quality.\n\nTry using lower quality settings, or use a better processing profile. You may also want to try using constant bitrate encoding." >> "$QualityReportFile"
  fi

fi 

echo "Quality Report File:"
echo ""

cat "$QualityReportFile"
