#!/bin/bash

# APPLICATION STARTS HERE

echo "YUVPrescaler is starting:" >> DebugOutput.txt

TargetNorm="$1"
TargetAspect="$2"

echo "YUVPrescaler target norm is $TargetNorm, target aspect is $TargetAspect." >> DebugOutput.txt

# Read info from stdout of decoder

read -u0 InputFrameRate
read -u0 InputSampleAspect
read -u0 InputXRes
read -u0 InputYRes
read -u0 InputSampleAspectX
read -u0 InputSampleAspectY
read -u0 VideoLength

echo "Received info on stdin:" >> DebugOutput.txt
echo "" >> DebugOutput.txt
echo "Input Framerate: $InputFrameRate" >> DebugOutput.txt
echo "Input Sample Aspect: $InputSampleAspect" >> DebugOutput.txt
echo "Input XRes: $InputXRes" >> DebugOutput.txt
echo "Input YRes: $InputYRes" >> DebugOutput.txt
echo "Input SampleAspectX: $InputSampleAspectX" >> DebugOutput.txt
echo "Input SampleAspectY: $InputSampleAspectY" >> DebugOutput.txt
echo "Video Length: $VideoLength ms" >> DebugOutput.txt
echo "" >> DebugOutput.txt

# Determine desired output sample aspect ratio and y4mscaler preset

if [ $TargetAspect == "standard" ]; then

  # Encode to 4:3 display aspect

  TargetDisplayAspect="4:3"
  ScalerPreset="DVD"

  if [ $TargetNorm == "PAL" ]; then

    TargetSampleAspect="59:54"
    TargetSampleAspectX=59
    TargetSampleAspectY=54

  else

    TargetSampleAspect="10:11"
    TargetSampleAspectX=10
    TargetSampleAspectY=11

  fi

else # Widescreen or Panavision

  # Encode to 16:9 display aspect

  TargetDisplayAspect="16:9"
  ScalerPreset="DVD_WIDE"

  if [ $TargetNorm == "PAL" ]; then

    TargetSampleAspect="118:81"
    TargetSampleAspectX=118
    TargetSampleAspectY=81

  else

    TargetSampleAspect="40:33"
    TargetSampleAspectX=40
    TargetSampleAspectY=33

  fi

fi

# Calculate the scaling parameters needed to fit the image
# in the full DVD frame, taking into account the sample aspect
# ratios of both input and output.

echo "Input resolution is ${InputXRes}x${InputYRes}" >> DebugOutput.txt

# True horiz. storage resolution of target MPEG2 file will be 720 pixels.
TargetXRes=720

# Now we need to calculate the vertical resolution based on source resolution,
# target resolution and pixel aspect ratios of both source and target video.

#echo "Scaling X to $TargetXRes" >> DebugOutput.txt

# Scale YRes exactly like XRes is scaled
#TempTargetYRes=$[ ( 720 * $InputYRes ) / $InputXRes ]
#echo "Scaling Y with the same factor to $TempTargetYRes" >> DebugOutput.txt

# If source did not have square pixels, translate to square-pixel resolution
#TempTargetYRes=$[ ( $TempTargetYRes * $InputSampleAspectY ) / $InputSampleAspectX ]
#echo "Compensate for non-square pixels in source, Y Res is now $TempTargetYRes" >> DebugOutput.txt

# When target MPEG video does not have square pixels, translate to non-square pixel resolution
#TempTargetYRes=$[ ( $TempTargetYRes * $TargetSampleAspectX ) / $TargetSampleAspectY ]
#echo "Compensate for non-square pixels in target, Y Res is now $TempTargetYRes" >> DebugOutput.txt

#TargetYRes=$TempTargetYRes

#Whole calculation in one go:

TargetYRes=$[ ( 720 * $InputYRes * $InputSampleAspectY * $TargetSampleAspectX ) / ($InputXRes * $InputSampleAspectX * $TargetSampleAspectY ) ]

# Make sure YRes is multiple of 8

TargetYRes=$[ ( $TargetYRes / 8 ) * 8 ]

TargetRes="${TargetXRes}x${TargetYRes}"


echo "Encoding to $TargetDisplayAspect aspect, storage resolution is $TargetRes" >> DebugOutput.txt


# Find clipping area in case YRes is larger than target picture size
# This may happen when converting 4:3 to 16:9 material, or when
# converting to letterboxed Panavision format.

if [ $TargetNorm == "PAL" ]; then

  if [ $TargetAspect == "letterboxed" ]; then

    MaxYRes=440

  else

    MaxYRes=576

  fi

else

  if [ $TargetAspect == "letterboxed" ]; then

    MaxYRes=360

  else

    MaxYRes=480

  fi

fi

if [ $TargetYRes -gt $MaxYRes ]; then

 echo "YRes greater than target image, video will be cropped." >> DebugOutput.txt

 InputActiveRegion="${TargetXRes}x${MaxYRes}+0+0cc"

else

 InputActiveRegion="${TargetXRes}x${TargetYRes}+0+0cc"

fi


if [ $TargetNorm == "PAL" ]; then

  ClipCommand="| y4mscaler -I norm=pal -I active=$InputActiveRegion -I matte=720x576+0+0cc -O preset=$ScalerPreset -O size=720x576"

else

  ClipCommand="| y4mscaler -I norm=ntsc -I active=$InputActiveRegion -I matte=720x480+0+0cc -O preset=$ScalerPreset -O size=720x480"

fi

# Scale picture to target resolution and SAR

if [ $TargetNorm == "PAL" ]; then

  #ScalerCommand="cat /dev/stdin | y4mscaler -I sar=$InputSampleAspect -O infer=exact -O infer=preserve_x -O size=$TargetRes -O sar=$TargetSampleAspect -S option=linear" 
  ScalerCommand="cat /dev/stdin | y4mscaler -I sar=$InputSampleAspect -O infer=exact -O infer=preserve_x -O size=$TargetRes -O sar=$TargetSampleAspect" 

else

  #ScalerCommand="cat /dev/stdin | y4mscaler -I sar=$InputSampleAspect -O infer=exact -O infer=preserve_x -O size=$TargetRes -O sar=$TargetSampleAspect -S option=linear" 
  ScalerCommand="cat /dev/stdin | y4mscaler -I sar=$InputSampleAspect -O infer=exact -O infer=preserve_x -O size=$TargetRes -O sar=$TargetSampleAspect" 

fi

# Pass on info to Filter / Postscaler / encoder

echo "$TargetAspect"
echo "$TargetNorm"
echo "$InputFrameRate"
echo "$ClipCommand"
echo "$VideoLength"

# Generate scaled YUV stream on stdout

echo "YUVPrescaler COMMAND: $ScalerCommand" >> DebugOutput.txt

eval "$ScalerCommand"